@@ -63,15 +63,15 @@ RDirNode::RDirNode(RDirNode* parent, std::string abspath) {
 
     visible_count = 0;
 
-    calcRadius();
-    calcColour();
-
     visible = false;
     position_initialized = false;
 
     since_node_visible = 0.0;
     since_last_file_change = 0.0;
     since_last_node_change = 0.0;
+
+    calcRadius();
+    calcColour();
 }
 
 void RDirNode::changePath(std::string abspath) {
@@ -118,6 +118,19 @@ void RDirNode::nodeUpdated(bool userInitiated) {
     if(parent !=0) parent->nodeUpdated(true);
 }
 
+void RDirNode::rotate(float s, float c) {
+
+    if(parent != 0) {
+        pos  = pos.rotate(s, c);
+        spos = spos.rotate(s, c);
+    }
+
+    for(std::list<RDirNode*>::iterator it = children.begin(); it != children.end(); it++) {
+        RDirNode* child = (*it);
+        child->rotate(s, c);
+    }
+}
+
 void RDirNode::setPos(vec2f pos) {
     this->pos = pos;
 }
@@ -823,7 +836,7 @@ void RDirNode::calcEdges() {
     }
 }
 
-void RDirNode::logic(float dt, Bounds2D& bounds) {
+void RDirNode::logic(float dt) {
 
     //move
     move(dt);
@@ -834,8 +847,6 @@ void RDirNode::logic(float dt, Bounds2D& bounds) {
         node_normal = (pos - parent->getPos()).normal();
     }
 
-    bounds.update(pos);
-
     //update files
      for(std::list<RFile*>::iterator it = files.begin(); it!=files.end(); it++) {
          RFile* f = *it;
@@ -847,7 +858,7 @@ void RDirNode::logic(float dt, Bounds2D& bounds) {
     for(std::list<RDirNode*>::iterator it = children.begin(); it != children.end(); it++) {
         RDirNode* node = (*it);
 
-        node->logic(dt, bounds);
+        node->logic(dt);
     }
 
     //update colour
