@@ -18,6 +18,7 @@
 #include "gource.h"
 
 float gGourceAutoSkipSeconds = 3.0;
+bool  gGourceDrawBackground  = true;
 bool  gGourceFileLoop        = false;
 bool  gGourceHideTree        = false;
 bool  gGourceHideUsernames   = false;
@@ -26,6 +27,8 @@ bool  gGourceDisableBloom    = false;
 bool  gGourceDisableProgress = false;
 bool  gGourceQuadTreeDebug   = false;
 
+int   gGourceMaxQuadTreeDepth = 6;
+
 std::string gGourceLogFormat;
 std::string gGourceDateFormat("%A, %d %B, %Y %X");
 
@@ -35,101 +38,26 @@ int gGourceMaxFiles = 1000;
 
 int gGourceUserInnerLoops = 0;
 
-#ifdef _WIN32
-HWND consoleWindow = 0;
-
-void createWindowsConsole() {
-    if(consoleWindow !=0) return;
-
-    //create a console on Windows so users can see messages
-
-    //find an available name for our window
-    int console_suffix = 0;
-    char consoleTitle[512];
-    sprintf(consoleTitle, "%s", "Gource Console");
-
-    while(FindWindow(0, consoleTitle)) {
-        sprintf(consoleTitle, "Gource Console %d", ++console_suffix);
-    }
-
-    AllocConsole();
-    SetConsoleTitle(consoleTitle);
-
-    //redirect streams to console
-    freopen("conin$", "r", stdin);
-    freopen("conout$","w", stdout);
-    freopen("conout$","w", stderr);
-
-    consoleWindow = 0;
-
-    //wait for our console window
-    while(consoleWindow==0) {
-        consoleWindow = FindWindow(0, consoleTitle);
-        SDL_Delay(100);
-    }
-
-    //disable the close button so the user cant crash gource
-    HMENU hm = GetSystemMenu(consoleWindow, false);
-    DeleteMenu(hm, SC_CLOSE, MF_BYCOMMAND);
-}
-#endif
-
-//info message
 void gource_info(std::string msg) {
-#ifdef _WIN32
-    createWindowsConsole();
-#endif
-
-    printf("%s\n", msg.c_str());
-
-#ifdef _WIN32
-    printf("\nPress Enter\n");
-    getchar();
-#endif
-
-    exit(0);
+    SDLAppInfo(msg);
 }
 
-//display error only
 void gource_quit(std::string error) {
-    SDL_Quit();
-
-#ifdef _WIN32
-    createWindowsConsole();
-#endif
-
-    printf("Error: %s\n\n", error.c_str());
-
-#ifdef _WIN32
-    printf("Press Enter\n");
-    getchar();
-#endif
-
-    exit(1);
+    SDLAppQuit(error);
 }
 
-//display help message + error (optional)
-void gource_help(std::string error) {
+//display help message
+void gource_help() {
 
 #ifdef _WIN32
-    createWindowsConsole();
+    SDLAppCreateWindowsConsole();
 
     //resize window to fit help message
-    if(consoleWindow !=0) {
-        RECT windowRect;
-        if(GetWindowRect(consoleWindow, &windowRect)) {
-            float width = windowRect.right - windowRect.left;
-            MoveWindow(consoleWindow,windowRect.left,windowRect.top,width,950,true);
-        }
-    }
+    SDLAppResizeWindowsConsole(950);
 #endif
 
     printf("Gource v%s\n", GOURCE_VERSION);
 
-    if(error.size()) {
-        printf("Error: %s\n\n", error.c_str());
-    }
-
     printf("Usage: gource [OPTIONS] [PATH]\n");
     printf("\nOptions:\n");
     printf("  -h, --help                       Help\n\n");
@@ -154,11 +82,12 @@ void gource_help(std::string error) {
     printf("  --colour-images                  Colourize user images\n\n");
 
     printf("  --date-format FORMAT     Specify display date string (strftime format)\n");
-    printf("  --log-format  FORMAT     Specify format of log (git,cvs,custom)\n");
+    printf("  --log-format  FORMAT     Specify format of log (git,cvs,hg,bzr,custom)\n");
     printf("  --git-branch             Get the git log of a particular branch\n");
     printf("  --git-log-command        Show git log command used by gource\n");
     printf("  --cvs-exp-command        Show cvs-exp.pl log command used by gource\n");
-    printf("  --hg-log-command         Show hg log (Mercurial) command used by gource\n\n");
+    printf("  --hg-log-command         Show hg log (Mercurial) command used by gource\n");
+    printf("  --bzr-log-command        Show bzr log (Bazaar) command used by gource\n\n");
 
     printf("  --multi-sampling         Enable multi-sampling\n");
     printf("  --crop AXIS              Crop view on an axis (vertical,horizontal)\n\n");
@@ -170,13 +99,7 @@ void gource_help(std::string error) {
     printf("  --disable-progress       Disable the progress bar\n");
     printf("  --disable-bloom          Disable bloom effect\n\n");
 
-    printf("  --hide-users             Hide users\n");
-    printf("  --hide-tree              Hide the tree\n");
-    printf("  --hide-files             Hide files\n");
-    printf("  --hide-usernames         Hide usernames\n");
-    printf("  --hide-filenames         Hide filenames\n");
-    printf("  --hide-dirnames          Hide directory names\n");
-    printf("  --hide-date              Hide the date\n\n");
+    printf("  --hide DISPLAY_ELEMENT   date,users,files,tree,usernames,filenames,dirnames\n\n");
 
     printf("  --max-files NUMBER       Max number of active files (default: 1000)\n");
     printf("  --max-file-lag SECONDS   Max time files of a commit can take to appear\n\n");
@@ -185,6 +108,8 @@ void gource_help(std::string error) {
     printf("  --user-friction SECONDS  Time users come to a complete hault (default: 0.67)\n");
     printf("  --user-scale SCALE       Change scale of users (default: 1.0)\n\n");
 
+    printf("  --camera-mode MODE       Camera mode (overview,track)\n\n");
+
     printf("  --follow-user USER       Camera will automatically follow this user\n");
     printf("  --highlight-user USER    Highlight the names of a particular user\n");
     printf("  --highlight-all-users    Highlight the names of all users\n");
@@ -193,7 +118,7 @@ void gource_help(std::string error) {
     printf("  --output-ppm-stream FILE Write frames as PPM to a file ('-' for STDOUT)\n");
     printf("  --output-framerate FPS   Framerate of output (25,30,60)\n\n");
 
-    printf("PATH may be a Git or Mercurial directory, a log file or '-' to read STDIN.\n");
+    printf("PATH may be a Git, Bazaar or Mercurial dir, a log file or '-' to read STDIN.\n");
     printf("If ommited, gource will attempt to generate a log from the current directory.\n\n");
 
 #ifdef _WIN32
@@ -201,12 +126,7 @@ void gource_help(std::string error) {
     getchar();
 #endif
 
-    //check if we should use an error code
-    if(error.size()) {
-        exit(1);
-    } else {
-        exit(0);
-    }
+    exit(0);
 }
 
 RCommitLog* Gource::determineFormat(std::string logfile) {
@@ -233,6 +153,11 @@ RCommitLog* Gource::determineFormat(std::string logfile) {
             if(clog->checkFormat()) return clog;
             delete clog;
         }
+        if(gGourceLogFormat == "bzr") {
+            clog = new BazaarLog(logfile);
+            if(clog->checkFormat()) return clog;
+            delete clog;
+        }
 
         if(gGourceLogFormat == "cvs") {
             clog = new CVSEXPCommitLog(logfile);
@@ -269,6 +194,13 @@ RCommitLog* Gource::determineFormat(std::string logfile) {
 
     delete clog;
 
+    //bzr
+    debugLog("trying bzr...\n");
+    clog = new BazaarLog(logfile);
+    if(clog->checkFormat()) return clog;
+
+    delete clog;
+
     //git raw
     debugLog("trying git raw...\n");
     clog = new GitRawCommitLog(logfile);
@@ -333,6 +265,7 @@ Gource::Gource(std::string logfile) {
     draw_loading = true;
 
     mousemoved = false;
+    mousedragged = false;
     mouseclicked = false;
 
     splash = -1.0;
@@ -353,6 +286,7 @@ Gource::Gource(std::string logfile) {
     date_x_offset = 0;
 
     camera = ZoomCamera(vec3f(0,0, -300), vec3f(0.0, 0.0, 0.0), 250.0, 5000.0);
+
     setCameraMode(false);
 
     root = 0;
@@ -373,8 +307,8 @@ Gource::Gource(std::string logfile) {
 Gource::~Gource() {
     reset();
 
-    delete commitlog;
-    delete root;
+    if(commitlog!=0) delete commitlog;
+    if(root!=0) delete root;
 
     //delete filters
     for(std::vector<Regex*>::iterator it = filters.begin(); it != filters.end(); it++) {
@@ -398,7 +332,7 @@ void Gource::update(float t, float dt) {
     dt *= time_scale;
 
     //have to manage runtime internally as we're messing with dt
-    runtime += dt;
+    if(!paused) runtime += dt;
 
     logic_time = SDL_GetTicks();
 
@@ -443,6 +377,29 @@ std::string Gource::dateAtPosition(float percent) {
 void Gource::mouseMove(SDL_MouseMotionEvent *e) {
     if(commitlog==0) return;
 
+    Uint8 ms = SDL_GetMouseState(0,0);
+    bool rightmouse = ms & SDL_BUTTON(SDL_BUTTON_RIGHT);
+
+    //move camera in direction the user dragged the mouse
+    if(mousedragged || rightmouse) {
+        vec2f mag( e->xrel, e->yrel );
+
+        if(mag.length()>100.0) return;
+
+        //if right mouse button is held while dragging, rotate tree instead of
+        //moving camera
+
+        if(rightmouse) {
+            rotate_angle = std::min(1.0f, (float) fabs(mag.x) / 10.0f) * 5.0f * DEGREES_TO_RADIANS;
+            if(mag.x < 0.0) rotate_angle = -rotate_angle;
+            return;
+        }
+
+        backgroundPos += mag;
+
+        return;
+    }
+
     mousepos = vec2f(e->x, e->y);
     mousemoved=true;
 
@@ -456,7 +413,6 @@ void Gource::mouseMove(SDL_MouseMotionEvent *e) {
 }
 
 void Gource::zoom(bool zoomin) {
-//    if(selectedFile == 0 && selectedUser == 0) return;
 
     float min_distance = camera.getMinDistance();
     float max_distance = camera.getMaxDistance();
@@ -477,7 +433,29 @@ void Gource::zoom(bool zoomin) {
 }
 
 void Gource::mouseClick(SDL_MouseButtonEvent *e) {
-    if(e->type != SDL_MOUSEBUTTONDOWN || commitlog==0) return;
+    if(commitlog==0) return;
+
+    if(e->type == SDL_MOUSEBUTTONUP) {
+
+        if(e->button == SDL_BUTTON_LEFT) {
+            //stop dragging mouse, return the mouse to where
+            //the user started dragging.
+            mousedragged=false;
+        }
+
+        Uint8 ms = SDL_GetMouseState(0,0);
+
+
+        if(e->button == SDL_BUTTON_LEFT || e->button == SDL_BUTTON_RIGHT) {
+            if(!(ms & SDL_BUTTON(SDL_BUTTON_RIGHT) || ms & SDL_BUTTON(SDL_BUTTON_LEFT))) {
+                SDL_WM_GrabInput(SDL_GRAB_OFF);
+                SDL_ShowCursor(true);
+                SDL_WarpMouse(mousepos.x, mousepos.y);
+            }
+        }
+    }
+
+    if(e->type != SDL_MOUSEBUTTONDOWN) return;
 
     //wheel up
     if(e->button == SDL_BUTTON_WHEELUP) {
@@ -491,14 +469,23 @@ void Gource::mouseClick(SDL_MouseButtonEvent *e) {
         return;
     }
 
-    if(e->button == SDL_BUTTON_RIGHT) {
+    if(e->button == SDL_BUTTON_MIDDLE) {
         toggleCameraMode();
         return;
     }
 
+
+    if(e->button == SDL_BUTTON_RIGHT) {
+
+        SDL_WM_GrabInput(SDL_GRAB_ON);
+        SDL_ShowCursor(false);
+
+        return;
+    }
+
     if(e->button == SDL_BUTTON_LEFT) {
 
-        mousepos = vec2f(e->x, e->y);
+        //mousepos = vec2f(e->x, e->y);
         mouseclicked=true;
 
         if(canSeek()) {
@@ -540,17 +527,40 @@ void Gource::setBackground(vec3f background) {
 void Gource::setCameraMode(bool track_users) {
     this->track_users = track_users;
     if(selectedUser!=0) camera.lockOn(track_users);
+    backgroundSelected=false;
 }
 
 void Gource::toggleCameraMode() {
     setCameraMode(!track_users);
 }
 
+//trace click of mouse on background
+void Gource::selectBackground() {
+    //is the left mouse button down?
+
+    Uint8 ms = SDL_GetMouseState(0,0);
+
+    if(!(ms & SDL_BUTTON(SDL_BUTTON_LEFT))) return;
+
+    selectUser(0);
+
+    backgroundSelected = true;
+
+    backgroundPos = camera.getPos().truncate();
+
+    SDL_ShowCursor(false);
+    SDL_WM_GrabInput(SDL_GRAB_ON);
+
+    mousedragged=true;
+}
+
 //select a user, deselect current file/user
 void Gource::selectUser(RUser* user) {
     //already selected do nothing
     if(user!=0 && selectedUser==user) return;
 
+    backgroundSelected=false;
+
     if(selectedFile != 0) {
         selectedFile->setSelected(false);
         selectedFile = 0;
@@ -582,6 +592,8 @@ void Gource::selectFile(RFile* file) {
     //already selected do nothing
     if(file!=0 && selectedFile==file) return;
 
+    backgroundSelected=false;
+
     if(selectedUser != 0) {
         selectedUser->setSelected(false);
         selectedUser = 0;
@@ -660,6 +672,20 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
             trace_debug = !trace_debug;
         }
 
+        if (e->keysym.sym == SDLK_m) {
+
+            Uint8 ms = SDL_GetMouseState(0,0);
+
+            //toggle mouse visiblity unless mouse clicked/pressed/dragged
+            if(!(mousedragged || mouseclicked || ms & SDL_BUTTON(SDL_BUTTON_LEFT))) {
+                if(SDL_ShowCursor(SDL_QUERY) == SDL_ENABLE) {
+                    SDL_ShowCursor(false);
+                } else {
+                    SDL_ShowCursor(true);
+                }
+            }
+        }
+
         if (e->keysym.sym == SDLK_n) {
             idle_time = gGourceAutoSkipSeconds;
         }
@@ -810,6 +836,13 @@ void Gource::reset() {
 
     selectedUser = 0;
     hoverUser = 0;
+    backgroundSelected=false;
+
+    mouseclicked=false;
+    mousemoved=false;
+    mousedragged = false;
+
+    rotate_angle = 0.0f;
 
     if(root!=0) delete root;
     root = new RDirNode(0, "/");
@@ -945,7 +978,7 @@ void Gource::readLog() {
     }
 
     if(first_read && commitqueue.size()==0) {
-        gource_quit("No commits found");
+        throw SDLAppException("no commits found");
     }
 
     first_read = false;
@@ -1106,7 +1139,7 @@ void Gource::interactUsers() {
 
     //dont use deep quad tree initially when all the nodes are in one place
     if(dir_bounds.area() > 10000.0) {
-        max_depth = 6;
+        max_depth = gGourceMaxQuadTreeDepth;
     }
 
     userTree = new QuadTree(quadtreebounds, max_depth, 1);
@@ -1152,6 +1185,29 @@ void Gource::interactUsers() {
     update_user_tree_time = SDL_GetTicks() - update_user_tree_time;
 }
 
+void Gource::updateBounds() {
+
+    user_bounds.reset();
+
+    for(std::map<std::string,RUser*>::iterator it = users.begin(); it!=users.end(); it++) {
+        RUser* u = it->second;
+
+        if(!u->isIdle()) {
+            user_bounds.update(u->getPos());
+        }
+    }
+
+    dir_bounds.reset();
+
+    for(std::map<std::string,RDirNode*>::iterator it = gGourceDirMap.begin(); it!=gGourceDirMap.end(); it++) {
+        RDirNode* node = it->second;
+
+        if(node->isVisible()) {
+            dir_bounds.update(node->getPos());
+        }
+    }
+}
+
 
 void Gource::updateUsers(float t, float dt) {
     std::vector<RUser*> inactiveUsers;
@@ -1179,8 +1235,6 @@ void Gource::updateUsers(float t, float dt) {
         if(u->isIdle()) {
             idle_users++;
         } else {
-            user_bounds.update(u->getPos());
-
             //if nothing is selected, and this user is active and this user is the specified user to follow, select them
             if(selectedUser == 0 && selectedFile == 0) {
                 for(std::vector<std::string>::iterator ui = follow_users.begin(); ui != follow_users.end(); ui++) {
@@ -1211,7 +1265,7 @@ void Gource::updateUsers(float t, float dt) {
     }
 }
 
-void Gource::interactDirs() {
+void Gource::updateQuadTree() {
 
     // update quad tree
     Bounds2D quadtreebounds = dir_bounds;
@@ -1227,7 +1281,7 @@ void Gource::interactDirs() {
 
     //dont use deep quad tree initially when all the nodes are in one place
     if(dir_bounds.area() > 10000.0) {
-        max_depth = 6;
+        max_depth = gGourceMaxQuadTreeDepth;
     }
 
     dirNodeTree = new QuadTree(quadtreebounds, max_depth, 1);
@@ -1243,15 +1297,11 @@ void Gource::interactDirs() {
     }
 
     update_dir_tree_time = SDL_GetTicks() - update_dir_tree_time;
-
-    root->applyForces(*dirNodeTree);
 }
 
 void Gource::updateDirs(float dt) {
-    //recalc the directory bounds
-    dir_bounds.reset();
-
-    root->logic(dt, dir_bounds);
+    root->applyForces(*dirNodeTree);
+    root->logic(dt);
 }
 
 void Gource::updateTime() {
@@ -1271,7 +1321,14 @@ void Gource::updateTime() {
 void Gource::updateCamera(float dt) {
 
     //camera tracking
-    if(track_users && (selectedFile !=0 || selectedUser !=0)) {
+
+    if(backgroundSelected) {
+        Bounds2D mousebounds;
+        mousebounds.update(backgroundPos);
+
+        camera.adjust(mousebounds);
+
+    } else if(track_users && (selectedFile !=0 || selectedUser !=0)) {
         Bounds2D focusbounds;
 
         vec3f camerapos = camera.getPos();
@@ -1296,17 +1353,24 @@ void Gource::logic(float t, float dt) {
 
     //init log file
     if(commitlog == 0) {
-        commitlog = determineFormat(logfile);
+
+        try {
+
+            commitlog = determineFormat(logfile);
+
+        } catch(SeekLogException& exception) {
+            throw SDLAppException("unable to read log file");
+        }
 
         if(commitlog == 0) {
             //if not in a git dir and no log file, show help
             if(logfile.size() == 0 || logfile == ".") {
                 SDL_Quit();
-                gource_help("");
+                throw SDLAppException("", true);
             } else if(SDLAppDirExists(logfile)) {
-                gource_quit("Directory not supported.");
+                throw SDLAppException("directory not supported");
             } else {
-                gource_quit("Unsupported log format.  You may need to regenerate your log file.");
+                throw SDLAppException("unsupported log format (you may need to regenerate your log file)");
             }
         }
 
@@ -1317,8 +1381,28 @@ void Gource::logic(float t, float dt) {
 
     slider.logic(dt);
 
+    //apply rotation
+    if(rotate_angle != 0.0f) {
+        float s = sinf(rotate_angle);
+        float c = cosf(rotate_angle);
+
+        root->rotate(s, c);
+
+        for(std::map<std::string,RUser*>::iterator it = users.begin(); it!=users.end(); it++) {
+            RUser* user = it->second;
+
+            vec2f userpos = user->getPos();
+
+            user->setPos(userpos.rotate(s, c));
+        }
+
+        rotate_angle = 0.0f;
+    }
+
     //still want to update camera while paused
     if(paused) {
+        updateBounds();
+        updateQuadTree();
         updateCamera(dt);
         return;
     }
@@ -1387,9 +1471,10 @@ void Gource::logic(float t, float dt) {
     gGourceFileInnerLoops = 0;
 
     interactUsers();
-    interactDirs();
-
     updateUsers(t, dt);
+
+    updateQuadTree();
+    updateBounds();
     updateDirs(dt);
 
     updateCamera(dt);
@@ -1407,12 +1492,12 @@ void Gource::mousetrace(Frustum& frustum, float dt) {
     glDepthFunc(GL_LEQUAL);
     glEnable(GL_DEPTH_TEST);
 
-	(void) glRenderMode(GL_SELECT);
+    (void) glRenderMode(GL_SELECT);
 
-	glInitNames();
-	glPushName(0);
+    glInitNames();
+    glPushName(0);
 
-	glMatrixMode(GL_PROJECTION);
+    glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
 
     gluPickMatrix((GLdouble) mousepos.x, (GLdouble) (viewport[3]-mousepos.y), 1.0f, 1.0f, viewport);
@@ -1509,11 +1594,14 @@ void Gource::mousetrace(Frustum& frustum, float dt) {
     if(mouseclicked) {
         if(hoverUser!=0) selectUser(hoverUser);
         else if(hoverFile!=0) selectFile(hoverFile);
-        else selectUser(0);
+        else {
+            selectBackground();
+        }
     }
 }
 
 void Gource::loadingScreen() {
+    if(!gGourceDrawBackground) return;
     display.mode2D();
 
     glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -1529,6 +1617,8 @@ void Gource::loadingScreen() {
 }
 
 void Gource::drawBackground(float dt) {
+    if(!gGourceDrawBackground) return;
+
     display.setClearColour(background_colour);
     display.clear();
 }
