@@ -18,7 +18,7 @@
 #ifndef GOURCE_H
 #define GOURCE_H
 
-#define GOURCE_VERSION "0.23"
+#define GOURCE_VERSION "0.26"
 
 #ifdef _WIN32
 #include "windows.h"
@@ -37,6 +37,7 @@
 
 #include "git.h"
 #include "hg.h"
+#include "bzr.h"
 #include "gitraw.h"
 #include "cvs-exp.h"
 #include "custom.h"
@@ -52,8 +53,10 @@
 #include "ppm.h"
 
 void createWindowsConsole();
+
+void gource_help();
 void gource_info(std::string msg);
-void gource_help(std::string error);
+void gource_quit(std::string error);
 
 class Gource : public SDLApp {
     std::string logfile;
@@ -74,8 +77,15 @@ class Gource : public SDLApp {
 
     bool mousemoved;
     bool mouseclicked;
+    bool mousedragged;
+
+    float rotate_angle;
+
     vec2f mousepos;
 
+    vec2f backgroundPos;
+    bool backgroundSelected;
+
     float start_position, stop_position;
     float last_percent;
     float time_scale;
@@ -149,6 +159,7 @@ class Gource : public SDLApp {
     void deleteUser(RUser* user);
     void deleteFile(RFile* file);
 
+    void selectBackground();
     void selectUser(RUser* user);
     void selectFile(RFile* file);
     void selectNextUser();
@@ -163,10 +174,12 @@ class Gource : public SDLApp {
     RCommitLog* determineFormat(std::string logfile);
 
     void interactUsers();
-    void interactDirs();
+
     void updateUsers(float t, float dt);
     void updateDirs(float dt);
     void updateCamera(float dt);
+    void updateQuadTree();
+    void updateBounds();
 
     void updateTime();
 
@@ -210,6 +223,7 @@ public:
 };
 
 extern float gGourceAutoSkipSeconds;
+extern bool  gGourceDrawBackground;
 extern bool  gGourceHideUsernames;
 extern bool  gGourceDisableBloom;
 extern bool  gGourceHideDate;
