@@ -37,17 +37,21 @@ int main(int argc, char *argv[]) {
     bool stop_on_idle = false;
     bool stop_at_end = false;
 
+    std::string camera_mode = "overview";
+
     std::string logfile = ".";
 
     std::vector<std::string> arguments;
 
+    SDLAppInit("Gource", "gource");
+
     SDLAppParseArgs(argc, argv, &width, &height, &fullscreen, &arguments);
 
     for(int i=0;i<arguments.size();i++) {
         std::string args = arguments[i];
 
         if(args == "-h" || args == "-?" || args == "--help") {
-            gource_help("");
+            gource_help();
         }
 
         if(args == "--loop") {
@@ -58,7 +62,7 @@ int main(int argc, char *argv[]) {
         if(args == "--git-branch") {
 
             if((i+1)>=arguments.size() || arguments[i+1].size() == 0) {
-                gource_help("specify a git branch");
+                gource_quit("specify a git branch");
             }
 
             gGourceGitLogCommand += " ";
@@ -81,11 +85,18 @@ int main(int argc, char *argv[]) {
                 gource_info(command);
         }
 
+        if(args == "--bzr-log-command") {
+                std::string command = gGourceBzrLogCommand();
+
+                gource_info(command);
+        }
+
+
 
         if(args == "--date-format") {
 
             if((i+1)>=arguments.size() || arguments[i+1].size() == 0) {
-                gource_help("specify a date format string");
+                gource_quit("specify a date format string");
             }
 
             gGourceDateFormat = arguments[++i];
@@ -93,11 +104,6 @@ int main(int argc, char *argv[]) {
             continue;
         }
 
-        if(args == "--hide-date") {
-            gGourceHideDate = true;
-            continue;
-        }
-
         if(args == "--disable-auto-skip") {
             gGourceAutoSkipSeconds = -1.0;
             continue;
@@ -113,33 +119,51 @@ int main(int argc, char *argv[]) {
             continue;
         }
 
-        if(args == "--hide-users") {
-            gGourceHideUsers = true;
-            continue;
+        std::vector<std::string> hide_fields;
+
+        if(args.find("--hide-") == 0 && args.size()>7) {
+
+            std::string hide = args.substr(7,args.size()-7);
+
+            hide_fields.push_back(hide);
         }
 
-        if(args == "--hide-tree") {
-            gGourceHideTree = true;
-            continue;
+        if(args == "--hide") {
+            if((i+1)>=arguments.size() || arguments[i+1].size() == 0) {
+                gource_quit("specify element(s) to hide");
+            }
+
+            std::string hidestr = arguments[++i];
+
+            size_t sep;
+            while((sep = hidestr.find(",")) != std::string::npos) {
+                if(sep!=0) hide_fields.push_back(hidestr.substr(0, sep));
+                if(sep == hidestr.size()-1) break;
+
+                hidestr = hidestr.substr(sep+1, hidestr.size()-1);
+            }
+
+            if(hidestr.size() && hidestr != ",") hide_fields.push_back(hidestr);
         }
 
-        if(args == "--hide-files") {
-            gGourceHideFiles = true;
-            continue;
-        }
+        if(hide_fields.size()>0) {
 
-        if(args == "--hide-usernames") {
-            gGourceHideUsernames = true;
-            continue;
-        }
+            for(std::vector<std::string>::iterator it = hide_fields.begin(); it != hide_fields.end(); it++) {
+                std::string hidestr = (*it);
 
-        if(args == "--hide-filenames") {
-            gGourceHideFilenames = true;
-            continue;
-        }
+                    if(hidestr == "date")      gGourceHideDate = true;
+                else if(hidestr == "users")     gGourceHideUsers = true;
+                else if(hidestr == "tree")      gGourceHideTree  = true;
+                else if(hidestr == "files")     gGourceHideFiles = true;
+                else if(hidestr == "usernames") gGourceHideUsernames = true;
+                else if(hidestr == "filenames") gGourceHideFilenames = true;
+                else if(hidestr == "dirnames")  gGourceDrawDirName = false;
+                else {
+                    std::string arg_error = std::string("unknown option --hide ") + hidestr;
+                    gource_quit(arg_error);
+                }
+            }
 
-        if(args == "--hide-dirnames") {
-            gGourceDrawDirName = false;
             continue;
         }
 
@@ -155,7 +179,7 @@ int main(int argc, char *argv[]) {
 
         if(args == "--crop") {
             if((i+1)>=arguments.size() || arguments[i+1].size() == 0) {
-                gource_help("specify crop (vertical,horizontal)");
+                gource_quit("specify crop (vertical,horizontal)");
             }
 
             std::string crop = arguments[++i];
@@ -165,7 +189,7 @@ int main(int argc, char *argv[]) {
             } else if (crop == "horizontal") {
                 gGourceHorizontalCrop = true;
             } else {
-                gource_help("invalid crop value");
+                gource_quit("invalid crop value");
             }
 
             continue;
@@ -173,7 +197,7 @@ int main(int argc, char *argv[]) {
 
         if(args == "--log-format") {
             if((i+1)>=arguments.size() || arguments[i+1].size() == 0) {
-                gource_help("specify log-format (format)");
+                gource_quit("specify log-format (format)");
             }
 
             gGourceLogFormat = arguments[++i];
@@ -181,8 +205,10 @@ int main(int argc, char *argv[]) {
             if(   gGourceLogFormat != "git"
                && gGourceLogFormat != "cvs"
                && gGourceLogFormat != "custom"
+               && gGourceLogFormat != "hg"
+               && gGourceLogFormat != "bzr"
                && gGourceLogFormat != "apache") {
-                gource_help("unknown log-format");
+                gource_quit("unknown log-format");
             }
 
             continue;
@@ -192,7 +218,7 @@ int main(int argc, char *argv[]) {
         if(args == "--default-user-image") {
 
             if((i+1)>=arguments.size() || arguments[i+1].size() == 0) {
-                gource_help("specify default-user-image (image path)");
+                gource_quit("specify default-user-image (image path)");
             }
 
             gGourceDefaultUserImage = arguments[++i];
@@ -203,7 +229,7 @@ int main(int argc, char *argv[]) {
         if(args == "--user-image-dir") {
 
             if((i+1)>=arguments.size() || arguments[i+1].size() == 0) {
-                gource_help("specify user-image-dir (directory)");
+                gource_quit("specify user-image-dir (directory)");
             }
 
             gGourceUserImageDir = arguments[++i];
@@ -218,13 +244,13 @@ int main(int argc, char *argv[]) {
 
         if(args == "--bloom-intensity") {
             if((i+1)>=arguments.size()) {
-                gource_help("specify bloom-intensity (float)");
+                gource_quit("specify bloom-intensity (float)");
             }
 
             gGourceBloomIntensity = atof(arguments[++i].c_str());
 
             if(gGourceBloomIntensity<=0.0) {
-                gource_help("invalid bloom-intensity value");
+                gource_quit("invalid bloom-intensity value");
             }
 
             continue;
@@ -233,13 +259,13 @@ int main(int argc, char *argv[]) {
         if(args == "--bloom-multiplier") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify bloom-multiplier (float)");
+                gource_quit("specify bloom-multiplier (float)");
             }
 
             gGourceBloomMultiplier = atof(arguments[++i].c_str());
 
             if(gGourceBloomMultiplier<=0.0) {
-                gource_help("invalid bloom-multiplier value");
+                gource_quit("invalid bloom-multiplier value");
             }
 
             continue;
@@ -248,13 +274,13 @@ int main(int argc, char *argv[]) {
         if(args == "-e" || args == "--elasticity") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify elasticity (float)");
+                gource_quit("specify elasticity (float)");
             }
 
             gGourceElasticity = atof(arguments[++i].c_str());
 
             if(gGourceElasticity<=0.0) {
-                gource_help("invalid elasticity value");
+                gource_quit("invalid elasticity value");
             }
 
             continue;
@@ -263,7 +289,7 @@ int main(int argc, char *argv[]) {
         if(args == "-b" || args == "--background") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify background colour (#FFFFFF)");
+                gource_quit("specify background colour (#FFFFFF)");
             }
 
             int r,g,b;
@@ -273,7 +299,7 @@ int main(int argc, char *argv[]) {
                 background = vec3f(r,g,b);
                 background /= 255.0f;
             } else {
-                gource_help("invalid colour string");
+                gource_quit("invalid colour string");
             }
 
             continue;
@@ -282,13 +308,13 @@ int main(int argc, char *argv[]) {
         if(args == "-s" || args == "--seconds-per-day") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify seconds-per-day (seconds)");
+                gource_quit("specify seconds-per-day (seconds)");
             }
 
             gGourceDaysPerSecond = atof(arguments[++i].c_str());
 
             if(gGourceDaysPerSecond<=0.0) {
-                gource_help("invalid seconds-per-day value");
+                gource_quit("invalid seconds-per-day value");
             }
 
             gGourceDaysPerSecond = 1.0 / gGourceDaysPerSecond;
@@ -305,13 +331,13 @@ int main(int argc, char *argv[]) {
         if(args == "-a" || args == "--auto-skip-seconds") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify auto-skip-seconds");
+                gource_quit("specify auto-skip-seconds");
             }
 
             gGourceAutoSkipSeconds = (float) atof(arguments[++i].c_str());
 
             if(gGourceAutoSkipSeconds <= 0.0) {
-                gource_help("invalid auto-skip-seconds value");
+                gource_quit("invalid auto-skip-seconds value");
             }
 
             continue;
@@ -320,13 +346,13 @@ int main(int argc, char *argv[]) {
         if(args == "-i" || args == "--file-idle-time") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify file-idle-time (seconds)");
+                gource_quit("specify file-idle-time (seconds)");
             }
 
             gGourceMaxFileIdle = (float) atoi(arguments[++i].c_str());
 
             if(gGourceMaxFileIdle<1.0) {
-                gource_help("invalid file-idle-time value");
+                gource_quit("invalid file-idle-time value");
             }
 
             continue;
@@ -335,13 +361,13 @@ int main(int argc, char *argv[]) {
         if(args == "-p" || args == "--start-position") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify start-position (float)");
+                gource_quit("specify start-position (float)");
             }
 
             start_position = atof(arguments[++i].c_str());
 
             if(start_position<=0.0 || start_position>=1.0) {
-                gource_help("start-position outside of range 0.0 - 1.0 (non-inclusive)");
+                gource_quit("start-position outside of range 0.0 - 1.0 (non-inclusive)");
             }
 
             continue;
@@ -350,13 +376,13 @@ int main(int argc, char *argv[]) {
         if(args == "--stop-position") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify stop-position (float)");
+                gource_quit("specify stop-position (float)");
             }
 
             stop_position = atof(arguments[++i].c_str());
 
             if(stop_position<=0.0 || stop_position>1.0) {
-                gource_help("stop-position outside of range 0.0 - 1.0 (inclusive)");
+                gource_quit("stop-position outside of range 0.0 - 1.0 (inclusive)");
             }
 
             continue;
@@ -379,13 +405,13 @@ int main(int argc, char *argv[]) {
         if(args == "--max-files") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify max-files (number)");
+                gource_quit("specify max-files (number)");
             }
 
             gGourceMaxFiles = atoi(arguments[++i].c_str());
 
             if(gGourceMaxFiles<1) {
-                gource_help("invalid max-files value");
+                gource_quit("invalid max-files value");
             }
 
             continue;
@@ -394,13 +420,13 @@ int main(int argc, char *argv[]) {
         if(args == "--max-file-lag") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify max-file-lag (seconds)");
+                gource_quit("specify max-file-lag (seconds)");
             }
 
             gGourceMaxFileLagSeconds = atof(arguments[++i].c_str());
 
             if(gGourceMaxFileLagSeconds==0.0) {
-                gource_help("invalid max-file-lag value");
+                gource_quit("invalid max-file-lag value");
             }
 
             continue;
@@ -409,13 +435,13 @@ int main(int argc, char *argv[]) {
         if(args == "--user-friction") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify user-friction (seconds)");
+                gource_quit("specify user-friction (seconds)");
             }
 
             gGourceUserFriction = atof(arguments[++i].c_str());
 
             if(gGourceUserFriction<=0.0) {
-                gource_help("invalid user-friction value");
+                gource_quit("invalid user-friction value");
             }
 
             gGourceUserFriction = 1.0 / gGourceUserFriction;
@@ -426,13 +452,13 @@ int main(int argc, char *argv[]) {
         if(args == "--user-scale") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify user-scale (scale)");
+                gource_quit("specify user-scale (scale)");
             }
 
             gGourceUserScale = atof(arguments[++i].c_str());
 
             if(gGourceUserScale<=0.0 || gGourceUserScale>100.0) {
-                gource_help("invalid user-scale value");
+                gource_quit("invalid user-scale value");
             }
 
             continue;
@@ -442,13 +468,13 @@ int main(int argc, char *argv[]) {
         if(args == "--max-user-speed") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify max-user-speed (units)");
+                gource_quit("specify max-user-speed (units)");
             }
 
             gGourceMaxUserSpeed = atof(arguments[++i].c_str());
 
             if(gGourceMaxUserSpeed<=0) {
-                gource_help("invalid max-user-speed value");
+                gource_quit("invalid max-user-speed value");
             }
 
             continue;
@@ -464,7 +490,7 @@ int main(int argc, char *argv[]) {
         if(args == "--highlight-user") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify highlight-user (user)");
+                gource_quit("specify highlight-user (user)");
             }
 
             highlight_users.push_back(arguments[++i]);
@@ -472,10 +498,26 @@ int main(int argc, char *argv[]) {
             continue;
         }
 
+        if(args == "--camera-mode") {
+
+            if((i+1)>=arguments.size()) {
+                gource_quit("specify camera-mode (overview,track)");
+            }
+
+            camera_mode = arguments[++i];
+
+            if(camera_mode != "overview" && camera_mode != "track") {
+                gource_quit("invalid camera-mode");
+            }
+
+            continue;
+        }
+
+
         if(args == "--follow-user") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify follow-user (user)");
+                gource_quit("specify follow-user (user)");
             }
 
             follow_users.push_back(arguments[++i]);
@@ -486,7 +528,7 @@ int main(int argc, char *argv[]) {
         if(args == "--file-filter") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify file-filter (regex)");
+                gource_quit("specify file-filter (regex)");
             }
 
             std::string filter_string = arguments[++i];
@@ -495,7 +537,7 @@ int main(int argc, char *argv[]) {
 
             if(!r->isValid()) {
                 delete r;
-                gource_help("invalid filt-filter regular expression");
+                gource_quit("invalid filt-filter regular expression");
             }
 
             filters.push_back(r);
@@ -506,14 +548,14 @@ int main(int argc, char *argv[]) {
         if(args == "--output-ppm-stream") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify ppm output file or '-' for stdout");
+                gource_quit("specify ppm output file or '-' for stdout");
             }
 
             ppm_file_name = arguments[++i];
 
 #ifdef _WIN32
             if(ppm_file_name == "-") {
-                gource_help("stdout PPM mode not supported on Windows");
+                gource_quit("stdout PPM mode not supported on Windows");
             }
 #endif
 
@@ -523,7 +565,7 @@ int main(int argc, char *argv[]) {
         if(args == "--output-framerate") {
 
             if((i+1)>=arguments.size()) {
-                gource_help("specify framerate (25,30,60)");
+                gource_quit("specify framerate (25,30,60)");
             }
 
             video_framerate = atoi(arguments[++i].c_str());
@@ -531,7 +573,7 @@ int main(int argc, char *argv[]) {
             if(   video_framerate != 25
                && video_framerate != 30
                && video_framerate != 60) {
-                gource_help("supported framerates are 25,30,60");
+                gource_quit("supported framerates are 25,30,60");
             }
 
             continue;
@@ -546,14 +588,14 @@ int main(int argc, char *argv[]) {
         // unknown argument
         std::string arg_error = std::string("unknown option ") + std::string(args);
 
-        gource_help(arg_error);
+        gource_quit(arg_error);
     }
 
     // wait for a character on the file handle if reading stdin
     if(logfile == "-") {
 
         if(gGourceLogFormat.size() == 0) {
-            gource_help("--log-format required when reading from STDIN");
+            gource_quit("--log-format required when reading from STDIN");
         }
 
         while(std::cin.peek() == EOF && !std::cin.fail()) SDL_Delay(100);
@@ -581,7 +623,7 @@ int main(int argc, char *argv[]) {
     }
 
     if(logfile.size()==0 || isdir) {
-        createWindowsConsole();
+        SDLAppCreateWindowsConsole();
     }
 #endif
 
@@ -590,47 +632,94 @@ int main(int argc, char *argv[]) {
         display.multiSample(4);
     }
 
+    //enable vsync
+    display.enableVsync(true);
+
+    try {
+
+        display.init("Gource", width, height, fullscreen);
+
+    } catch(SDLInitException& exception) {
+
+        char errormsg[1024];
+        snprintf(errormsg, 1024, "SDL initialization failed - %s", exception.what());
+
+        gource_quit(errormsg);
+    }
+
     //init frame exporter
     FrameExporter* exporter = 0;
 
     if(ppm_file_name.size() > 0) {
-        exporter = new PPMExporter(ppm_file_name);
+        //recording a video kind of implies you want this
+        stop_at_end = true;
+
+        try {
+
+            exporter = new PPMExporter(ppm_file_name);
+
+        } catch(PPMExporterException& exception) {
+
+            char errormsg[1024];
+            snprintf(errormsg, 1024, "could not write to '%s'", exception.what());
+
+            gource_quit(errormsg);
+        }
     }
 
-    //enable vsync
-    display.enableVsync(true);
-
-    display.init("Gource", width, height, fullscreen);
-
     if(multisample) glEnable(GL_MULTISAMPLE_ARB);
 
-    Gource* gource = new Gource(logfile);
+    Gource* gource = 0;
 
-    if(start_position>0.0) gource->setStartPosition(start_position);
-    if(stop_position>0.0)  gource->setStopPosition(stop_position);
+    try {
+        gource = new Gource(logfile);
 
-    gource->setStopAtEnd(stop_at_end);
-    gource->setStopOnIdle(stop_on_idle);
+        if(camera_mode == "track") {
+            gource->setCameraMode(true);
+        }
 
-    if(exporter!=0) gource->setFrameExporter(exporter, video_framerate);
+        if(start_position>0.0) gource->setStartPosition(start_position);
+        if(stop_position>0.0)  gource->setStopPosition(stop_position);
 
-    gource->setBackground(background);
+        gource->setStopAtEnd(stop_at_end);
+        gource->setStopOnIdle(stop_on_idle);
+
+        if(exporter!=0) gource->setFrameExporter(exporter, video_framerate);
+
+        gource->setBackground(background);
+
+        for(std::vector<std::string>::iterator it = follow_users.begin(); it != follow_users.end(); it++) {
+            gource->addFollowUser(*it);
+        }
+
+        for(std::vector<std::string>::iterator it = highlight_users.begin(); it != highlight_users.end(); it++) {
+            gource->addHighlightUser(*it);
+        }
+
+        for(std::vector<Regex*>::iterator it = filters.begin(); it != filters.end(); it++) {
+            gource->addFilter(*it);
+        }
+
+        gource->run();
+
+    } catch(ResourceException& exception) {
+
+        char errormsg[1024];
+        snprintf(errormsg, 1024, "failed to load resource '%s'", exception.what());
+
+        gource_quit(errormsg);
+
+    } catch(SDLAppException& exception) {
+
+        if(exception.showHelp()) {
+            gource_help();
+        } else {
+            gource_quit(exception.what());
+        }
 
-    for(std::vector<std::string>::iterator it = follow_users.begin(); it != follow_users.end(); it++) {
-        gource->addFollowUser(*it);
     }
 
-    for(std::vector<std::string>::iterator it = highlight_users.begin(); it != highlight_users.end(); it++) {
-        gource->addHighlightUser(*it);
-    }
-
-    for(std::vector<Regex*>::iterator it = filters.begin(); it != filters.end(); it++) {
-        gource->addFilter(*it);
-    }
-
-    gource->run();
-
-    delete gource;
+    if(gource!=0) delete gource;
 
     if(exporter != 0) delete exporter;
 
